package ch.SWITCH.aai.uApprove.storage;

import ch.SWITCH.aai.uApprove.components.ConsentMode;
import ch.SWITCH.aai.uApprove.components.UApproveException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import java.util.HashSet;
import java.util.TreeMap;
import java.util.Set;
import java.util.Collections;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * Class: LogInfo
 * 
 * Copyright (c) 2011, National Insutitute of Informatics
 * Copyright (c) 2005-2006 SWITCH - The Swiss Education & Research Network
 * 
 * 
 * 
 * @author poroli
 *         <p>
 *         The class LogInfo holds a set of UserLogInfo classes. In the class
 *         UserLogInfo we record the version number approved by a user.
 *         </p>
 * 
 *         This is the original implementation of LogInfo, which was damned to
 *         be a subclass of LogInfo, when the support for JDBC was added.
 * 
 * @author C.Witzig 1.3.2006
 * 
 * 
 * 
 *         Modifications: -------------- 1. Added proper exception handling in
 *         LogInfoFile.initialize if the file couldn't be created chw 18.6.2007
 * 
 */

public class LogInfoFile extends LogInfo {
  private static LogInfoFile logInfo;

  private static String filename;

  private static File userLogInfoFile = null;

  private Map<String, UserLogInfo> data;

  private static Logger LOG = LoggerFactory.getLogger(LogInfoFile.class);

  /**
   * Singleton Constructor
   */
  private LogInfoFile() {
    data = new HashMap<String, UserLogInfo>();
  }

  /**
   * Singleton getInstance
   */
  public static synchronized LogInfo getInstance(String filename)
      throws UApproveException {
    LOG.info("LogInfoFile.getInstance(" + filename + ")");
    logInfo = new LogInfoFile();
    LogInfoFile.filename = filename;
    logInfo.initialize();
    return logInfo;
  }

  /**
   * 
   * read data from xml file into hashmap
   */
  private void initialize() throws UApproveException {
    try {
      userLogInfoFile = new File(filename);
      if (!userLogInfoFile.exists()) {
        OutputStreamWriter out = new OutputStreamWriter(new FileOutputStream(
            userLogInfoFile, false));
        out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
        out.write("<Rule>");
        out.write("</Rule>");
        out.flush();
        out.close();
      }
    } catch (IOException e) {
      e.printStackTrace();
      throw new UApproveException(
          "LogInfoFile.initialize: cannot open or write file [" + filename
              + "]");
    }
    Document userSettingDoc = createDocument();
    logInfo.readFromXmlDocument(userSettingDoc);
  }

  private Document createDocument() throws UApproveException {
    Document userSettingDoc = null;
    try {
      DocumentBuilder db = DocumentBuilderFactory.newInstance()
          .newDocumentBuilder();
      userSettingDoc = db.parse(new FileInputStream(userLogInfoFile));
    } catch (Exception e) {
      throw new UApproveException(
          "LogInfoFile:initialize: An exception occurred trying to read the file ["
              + filename + "]");
    }
    return userSettingDoc;
  }

  /**
   * Read the user settings LogInfo XML document
   * 
   * @param doc
   * @throws UApproveException
   */
  private void readFromXmlDocument(Document doc) throws UApproveException {
    LOG.trace("XML is re-readed");
    Node n, t;
    String username, version, date, termsVersion;
    String global = "no";
    NodeList userList;

    NodeList list = doc.getElementsByTagName("User");

    for (int i = 0, lasti = list.getLength(); i < lasti; i++) {
      // Set<String> setProviderIds = Collections.synchronizedSortedSet( new
      // TreeSet<String>() );
      Map<String, String> mapProviderIds = Collections.synchronizedSortedMap(new TreeMap<String, String>());
      Set<String> checkProviderIds = Collections.synchronizedSet(new HashSet<String>());
      version = date = termsVersion = null;
      n = list.item(i);
      t = n.getAttributes().getNamedItem("name");
      if (t == null)
        throw new UApproveException("In file [" + filename
            + "] file found a <User> tag without the attribute \"name\"");
      username = t.getNodeValue();
      userList = n.getChildNodes();
      for (int j = 0, lastj = userList.getLength(); j < lastj; j++) {
        n = userList.item(j);
        if (n.getNodeName().equals("Version")) {
          t = n.getAttributes().getNamedItem("id");
          if (t == null) {
            version = ""; // version can be left out when not using user
          } else {
            version = t.getNodeValue();
          }
          t = n.getAttributes().getNamedItem("terms-id");
          if (t == null) {
            termsVersion = ""; // termsVersion is not needed
          } else {
            termsVersion = t.getNodeValue();
          }
          t = n.getAttributes().getNamedItem("installationdate");
          if (t == null)
            throw new UApproveException(
                "In file ["
                    + filename
                    + "] found a <Version> tag without the \"installationdate\" attribute.");
          date = t.getNodeValue();
          t = n.getAttributes().getNamedItem("global");
          global = (t == null) ? "no" : t.getNodeValue();
        }
        if (n.getNodeName().equals("providerId")) {
          Node nodeProviderName = n.getAttributes().getNamedItem("name");
          LOG.debug("providerId node: " + (nodeProviderName != null ? nodeProviderName.getNodeValue() : "?"));
          if (nodeProviderName != null) {
              String theProviderId = nodeProviderName.getNodeValue();

              boolean checkalways = false;
              t = n.getAttributes().getNamedItem("checkalways");
              LOG.debug("checkalways attr: " + (t != null ? t.getNodeValue() : "null"));
              if (t != null && t.getNodeValue().equals("yes")) {
            	LOG.debug("checkalways {} is yes", theProviderId);
            	checkalways = true;
              	if (!checkProviderIds.contains(theProviderId)) {
              		checkProviderIds.add(theProviderId);
                	LOG.debug("checkProviderIds added: {}", theProviderId);
              	}
              	else {
                	LOG.debug("checkProviderIds contained: {}", theProviderId);
              	}
              }
              else {
              	LOG.debug("checkalways {} is no", theProviderId);
              }
        	  Node nodeAttributes = n.getAttributes().getNamedItem("attributes");
            String sAttributes = (nodeAttributes != null) ? nodeAttributes
                .getNodeValue() : (checkalways ? null : "");
            mapProviderIds.put(theProviderId, sAttributes);

          }
        }
      }

      if (LOG.isDebugEnabled()) {
        LOG.debug("LogInfo.readFromXmlDocument: user = " + username
            + " version = " + version + " date = " + date + " termsVersion = "
            + termsVersion + " global " + global
            + " mapProviderIds=" + mapProviderIds + " checkProviderIds=" + checkProviderIds);
      }

      if (username == null || username == "")
        throw new UApproveException("Could not read the username in the file "
            + filename + ".");

      if (date == null || date == "")
        throw new UApproveException(
            "Could not detect the install date in the file " + filename + ".");

      this.addUserLogInfoData(username, version, date, termsVersion, global,
          mapProviderIds, checkProviderIds);
    }
  }

  /**
   * Returns the data for all the users.
   * 
   * @return Map
   */
  public Map<String, UserLogInfo> getData() {

    return logInfo.data;
  }

  /**
   * Returns the data related to a user.
   * 
   * @param username
   * @return UserLogInfo
   */
  public synchronized UserLogInfo getData(String username) {

    LOG.info("re read data file:" + filename);
    try {
      logInfo.data.clear();
      Document userSettingDoc = createDocument();
      readFromXmlDocument(userSettingDoc);
    } catch (UApproveException e) {
      LOG.error("Exception ", e);
      return null;
    }

    if (LOG.isDebugEnabled()) {
      StringBuilder sb = new StringBuilder();
      sb.append("getData(").append(username).append(")\n");
      sb.append("  data=").append(data).append("\n");
      sb.append("  loginfo.data=").append(logInfo.data).append("\n");
      LOG.debug(sb.toString());
    }

    if (logInfo.data.containsKey(username)) {
      return (UserLogInfo) logInfo.data.get(username);
    }
    return null;
  }
  
  public UserLogInfo getDataSimple(String username) {
	  return getData(username);
  }

  /**
   * Write LogInfo to xml file
 * @param session
   * 
   * @throws UApproveException
   */
  public synchronized void update(UserLogInfo theUserData,
			String sProviderId, ConsentMode cm)
		throws UApproveException {
    if (logInfo == null) {
      throw new UApproveException(
          "LogInfo was not yet initialized and now it need to be saved????.");
    }

    if (theUserData != null) {
    	LOG.debug("update logInfo.data (" + theUserData.getUsername() + ")");
    	Map<String, UserLogInfo> map = getData();
    	map.put(theUserData.getUsername(), theUserData);
    }

    File f = new File(filename);
    OutputStreamWriter out = null;
    try {
      out = new OutputStreamWriter(new FileOutputStream(f, false));
      out.write("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
      out.write("<Users>\n");
      UserLogInfo uLogInfo;
      Iterator<String> iter = getData().keySet().iterator();
      // write all User elements
      while (iter.hasNext()) {
        uLogInfo = (UserLogInfo) getData().get(iter.next());
        out.write("  <User name=\"" + uLogInfo.getUsername() + "\">\n");
        out.write("    <Version id=\"" + uLogInfo.getVersion() + "\"");
        if (!(uLogInfo.getTermsVersion() == null || uLogInfo.getTermsVersion()
            .equals(""))) {
          out.write(" terms-id=\"" + uLogInfo.getTermsVersion() + "\"");
        }
        out.write(" global=\"" + uLogInfo.getGlobal() + "\"");
        out.write(" installationdate=\"" + uLogInfo.getOndate() + "\"/>\n");

        // write out the providers if any
        // Set<String> setProviderIds = uLogInfo.getProviderIds();
        Map<String, String> mapProviderIds = uLogInfo.getProviderIds();
        LOG.debug("mapProviderIds is " + (mapProviderIds.isEmpty() ? "empty" : "not empty"));        		
        Set<String> keySet = mapProviderIds.keySet();
        LOG.debug("keySet is " + (keySet.isEmpty() ? "empty" : "not empty"));
        if (!keySet.isEmpty()) {
        	for (String key : keySet) {
        	LOG.debug("  " + key);
        	}
        }
        Iterator<String> it = keySet.iterator();
        while (it.hasNext()) {
          String theKey = (String) it.next();
          String theAttr = (String) mapProviderIds.get(theKey);
          String checkAlways = uLogInfo.checkAlways(theKey) ? "yes" : "no";
		out.write("         <providerId name=\"" + theKey
              + "\" attributes=\"" + theAttr
              + "\" checkalways=\"" + checkAlways + "\"/>\n");
        LOG.debug("         <providerId name=\"" + theKey
                + "\" attributes=\"" + theAttr
                + "\" checkalways=\"" + checkAlways + "\"/>");
        }
        out.write("  </User>\n");
        if (LOG.isDebugEnabled()) {
          LOG.debug("LogInfo.update: writing user " + uLogInfo.getUsername()
              + " version = " + uLogInfo.getVersion() + " date = "
              + uLogInfo.getOndate() + " termsVersion = "
              + uLogInfo.getTermsVersion() + " global " + uLogInfo.getGlobal()
              + " mapProviderIds=" + uLogInfo.getProviderIds()
              + " checkProviderIds=" + uLogInfo.getCheckAlwaysProviderIds());
        }
      }
      out.write("</Users>\n");
      out.flush();
    } catch (IOException ioe) {
      throw new UApproveException(
          "An IOException occurred trying to write the loginfo.xml file ["
              + filename + "].");
    } finally {
      try {
        out.close();
      } catch (IOException e) {
        e.printStackTrace();
      }
    }
  }

  public UserLogInfo addUserLogInfoData(String username, String version,
      String date, String termsVersion, String global, String providerId,
      String attributesReleased) {
    UserLogInfo userInfo = new UserLogInfo(username, version, date,
        termsVersion, global);
    if (providerId != null && attributesReleased != null)
      userInfo.addProviderId(providerId, attributesReleased);
    logInfo.data.put(username, userInfo);
    return userInfo;
  }

  public void addUserLogInfoData(String username, String version, String date,
      String termsVersion, String sGlobal, Map<String, String> mapProviderIds,
      Set<String> checkAlways) {
    logInfo.data.put(username, new UserLogInfo(username, version, date,
        termsVersion, sGlobal, mapProviderIds, checkAlways));
  }

  @Override
  public synchronized void cleanupAttrsApproval(String theUsername, String theProvider)
			throws UApproveException {
	  LOG.info("cleanup check always provider record. " + theUsername);

	  if (!logInfo.data.containsKey(theUsername)) {
		  LOG.info("user not found:" + theUsername);
		  return;
	  }
	  UserLogInfo userLogInfo = logInfo.data.get(theUsername);
	  if (userLogInfo.checkAlways(theProvider)) {
	    if (LOG.isDebugEnabled()) {
	      LOG.debug("check always == yes: the provider {}", theProvider);
	      printProviderIds(userLogInfo);
	    }
      Map<String, String> providerIds = userLogInfo.getProviderIds();
		  if (providerIds.containsKey(theProvider)) {
			  LOG.debug("clean attributes the provider");
			  //providerIds.put(theProvider, null);// ?
			  providerIds.remove(theProvider);
			  LOG.info("update file.");
			  logInfo.update(userLogInfo);
		  }
	  }
	  else {
		  LOG.debug("check always != yes: the provider {}", theProvider);
	  }
  }

  private void printProviderIds(UserLogInfo userLogInfo) {
    Map<String, String> providerIds = userLogInfo.getProviderIds();
    StringBuilder sb = new StringBuilder("*** providerIds:");
    for (String key : providerIds.keySet()) {
      sb.append(key);
      sb.append(", ");
    }
    LOG.debug(sb.toString());
  }

// not implemented in LogInfoFile subclass
  public synchronized void updateProviderAccess(String theUsername,
      String theProvider, boolean bGlobal) throws UApproveException {
    return;
  }

  // not implemented in LogInfoFile subclass
  public synchronized void updateProviderAccessWithNoARA(String theUsername,
      String theProvider) throws UApproveException {
    return;
  }

  public synchronized String getUserReleaseAttrs(String theUsername,
      String theProvider, boolean bGlobal) throws UApproveException {
    UserLogInfo userInfo = logInfo.data.get(theUsername);

    String global;
    if (bGlobal) {
        global = "yes";
    } else {
        global = "no";
    }
    if (userInfo != null && userInfo.getGlobal().equals(global)) {
        return userInfo.getAttributesForProviderId(theProvider);
    } else {
        return null;
    }
  }

}
