package ch.SWITCH.aai.uApprove.storage;

import java.util.Collections;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;


/**
 * Class that contains the info for one user.
 * 
 * Copyright (c) 2011, National Insutitute of Informatics
 * Copyright (c) 2005-2006 SWITCH - The Swiss Education & Research Network
 * 
 * 
 * Purpose: store all info for a user in a class and keep it in memory, such
 * that it can be retrieved very quickly.
 * 
 * Developped based on the original version from F.Poroli. Addition of global
 * and mapProviderIds parameter
 * 
 * @author C.Witzig 8.12.2005
 * 
 */

public class UserLogInfo {
  private String username; // user's name
  private String version; // version of setting accepted
  private String ondate; // date when last setting applied
  private String termsVersion; // terms of use version accepted
  private String global; // whether the user wants to have a global approval
  private Map<String, String> mapProviderIds;
  private Set<String> checkProviderIds;
  
  /**
   * 
   * @param username
   * @param version
   * @param ondate
   */
  public UserLogInfo(String username, String version, String ondate) {
    this(username, version, ondate, "", "no");
  }

  public UserLogInfo(String username, String version, String ondate,
      String termsVersion) {
    this(username, version, ondate, termsVersion, "no");
  }

  /**
   * 
   * @param username
   * @param version
   * @param ondate
   * @param termsVersion
   * @param sGlobal
   */
  public UserLogInfo(String username, String version, String ondate,
      String termsVersion, String sGlobal) {
    this.username = username;
    this.version = version;
    this.ondate = ondate;
    this.termsVersion = termsVersion;
    this.global = sGlobal;
    // setProviderIds = Collections.synchronizedSortedSet( new TreeSet<String>()
    // );
    mapProviderIds = Collections.synchronizedSortedMap(new TreeMap<String, String>());
    checkProviderIds = Collections.synchronizedSet(new HashSet<String>());
  }

  public UserLogInfo(String username, String version, String ondate,
      String termsVersion, String sGlobal, Map<String, String> mapProviderIds, Set<String> checkProvideIds) {
    this.username = username;
    this.version = version;
    this.ondate = ondate;
    this.termsVersion = termsVersion;
    this.global = sGlobal;
    this.mapProviderIds = mapProviderIds;
    this.checkProviderIds = checkProvideIds;
  }

  public void setUsername(String username) {
    this.username = username;
  }

  public void setVersion(String version) {
    this.version = version;
  }

  public void setOndate(String ondate) {
    this.ondate = ondate;
  }

  public void setTermsVersion(String version) {
    this.termsVersion = version;
  }

  public void setGlobal(String sGlobal) {
    this.global = sGlobal;
  }

  public String getUsername() {
    return username;
  }

  public String getVersion() {
    return version;
  }

  public String getOndate() {
    return ondate;
  }

  public String getTermsVersion() {
    return termsVersion != null ? termsVersion : "";
  }

  public String getGlobal() {
    return global;
  }

  public void addProviderId(String sProviderId, String sAttributes) {
    mapProviderIds.put(sProviderId, sAttributes);
  }

  public boolean containsProviderId(String sProviderId) {
    return mapProviderIds.containsKey(sProviderId);
  }

  public boolean containsAttributeForProviderId(String sProviderId,
      String sAttribute, String sDelimiter) {
    String theAttributes = mapProviderIds.get(sProviderId);
    if (theAttributes == null)
      return false;

    return theAttributes.indexOf(sDelimiter + sAttribute + sDelimiter) != -1 ? true
        : false;
  }

  public String getAttributesForProviderId(String sProviderId) {
    return mapProviderIds.get(sProviderId);
  }

  public Map<String, String> getProviderIds() {
    return mapProviderIds;
  }

  public boolean checkAlways(String providerId) {
	  return checkProviderIds.contains(providerId);
  }

  public Set<String> getCheckAlwaysProviderIds() {
	  return checkProviderIds;
  }

  public void addCheckAlways(String providerId) {
	  checkProviderIds.add(providerId);
  }

  public void clearCheckAlways(String providerId) {
	  checkProviderIds.remove(providerId);
  }

  public void clearRelease() {
    mapProviderIds.clear();
    checkProviderIds.clear();
  }
  
  public void clearRelease(String providerId) {
    mapProviderIds.remove(providerId);
  }
  

  public String toString() {
	return "username=" + username + ", global=" + global + ", mapProviderIds="
			+ mapProviderIds + ", ondate=" + ondate + ", termsVersion="
			+ termsVersion + ", checkAlways=" + checkProviderIds;
  }

  
}
