package ch.SWITCH.aai.uApprove.idpplugin;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import org.opensaml.saml2.core.NameID;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import ch.SWITCH.aai.uApprove.components.Attribute;
import ch.SWITCH.aai.uApprove.components.UApproveException;
import edu.internet2.middleware.shibboleth.common.attribute.AttributeRequestException;
import edu.internet2.middleware.shibboleth.common.attribute.BaseAttribute;
import edu.internet2.middleware.shibboleth.common.attribute.encoding.AttributeEncoder;
import edu.internet2.middleware.shibboleth.common.attribute.encoding.SAML2AttributeEncoder;
import edu.internet2.middleware.shibboleth.common.attribute.provider.SAML1AttributeAuthority;
import edu.internet2.middleware.shibboleth.common.attribute.provider.SAML2AttributeAuthority;
import edu.internet2.middleware.shibboleth.common.attribute.provider.ScopedAttributeValue;
import edu.internet2.middleware.shibboleth.common.profile.provider.BaseSAMLProfileRequestContext;
import edu.internet2.middleware.shibboleth.idp.profile.saml1.BaseSAML1ProfileRequestContext;
import edu.internet2.middleware.shibboleth.idp.profile.saml2.BaseSAML2ProfileRequestContext;

public class AttributeDumper {
	private static Logger LOG = LoggerFactory.getLogger(AttributeDumper.class);

	private static AttributeDumper dumper;

	private static SAML1AttributeAuthority saml1AA;
	private static SAML2AttributeAuthority saml2AA;
	// create the dumper instance and initialize it
	private AttributeDumper(SAML1AttributeAuthority saml1AA, SAML2AttributeAuthority saml2AA) {
		AttributeDumper.saml1AA = saml1AA;
		AttributeDumper.saml2AA = saml2AA;
	}

	public static void initialize(SAML1AttributeAuthority saml1AA, SAML2AttributeAuthority saml2AA) {
		if (dumper == null) {
			dumper = new AttributeDumper(saml1AA, saml2AA);
		}
		LOG.debug("AttributeDumper initialized");
	}

	@SuppressWarnings({ "unchecked", "rawtypes" })
	public static Collection<Attribute> getAttributes(BaseSAMLProfileRequestContext requestCtx) throws UApproveException {
		if (dumper == null) {
			throw new UApproveException("AttributeDumper is not initializied");
		}
		if (LOG.isDebugEnabled()) {
			LOG.debug("requestCtx is {}", requestCtx.getClass().getSimpleName());
		}

		try {
			Map<String, BaseAttribute> attributes = saml2AA.getAttributes(requestCtx);
			return AttributeDumper.getAttributes(attributes);
		} catch (AttributeRequestException e) {
			LOG.error("Unable to retrieve attributes Message", e);
			throw new UApproveException("Unable to retrieve attributes Message", e);
		}
	}

	@SuppressWarnings("rawtypes")
	public static Collection<Attribute> getAttributes(BaseSAML1ProfileRequestContext<?, ?, ?> requestCtx) throws UApproveException {
		if (dumper == null) {
			throw new UApproveException("AttributeDumper is not initializied");
		}
		LOG.debug("requestCtx is BaseSAML1ProfileRequestContext");

		try {
			Map<String, BaseAttribute> attributes = saml1AA.getAttributes(requestCtx);
			return AttributeDumper.getAttributes(attributes);
		} catch (AttributeRequestException e) {
			LOG.error("Unable to retrieve attributes Message", e);
			throw new UApproveException("Unable to retrieve attributes Message", e);
		}
	}

	@SuppressWarnings("rawtypes")
	public static Collection<Attribute> getAttributes(BaseSAML2ProfileRequestContext<?, ?, ?> requestCtx) throws UApproveException {
		if (dumper == null) {
			throw new UApproveException("AttributeDumper is not initializied");
		}
		LOG.debug("requestCtx is BaseSAML2ProfileRequestContext");

		try {
			Map<String, BaseAttribute> attributes = saml2AA.getAttributes(requestCtx);
			return AttributeDumper.getAttributes(attributes);
		} catch (AttributeRequestException e) {
			LOG.error("Unable to retrieve attributes Message", e);
			throw new UApproveException("Unable to retrieve attributes Message", e);
		}
	}

	@SuppressWarnings({ "rawtypes" })
	private static Collection<Attribute> getAttributes(Map<String, BaseAttribute> attributes) {
		Collection<Attribute> result = new ArrayList<Attribute>();
		LOG.trace("DIRECT OUTPUT FROM RESOLVING");
		for (BaseAttribute<?> attr : attributes.values()) {

			Collection<String> attributeValues = new ArrayList<String>();
			for (Iterator<?> iter = attr.getValues().iterator(); iter.hasNext();) {
				Object valueObj = iter.next();
				if (valueObj != null && !valueObj.toString().trim().equals("")) {
					LOG.trace(valueObj.toString());
					if (valueObj instanceof NameID) {
						NameID nameIdObject = (NameID)valueObj;
						attributeValues.add(nameIdObject.getValue());
					} else if (valueObj instanceof ScopedAttributeValue) {
						ScopedAttributeValue scopedAttr = (ScopedAttributeValue)valueObj;
						attributeValues.add(scopedAttr.getValue() + "@" + scopedAttr.getScope());
					} else {
						attributeValues.add(valueObj.toString());
					}
				}
			}

			LOG.debug("Attribute '{}' has {} value(s)", attr.getId(),attributeValues.size());

			if (attributeValues.size() == 0) {
				continue;
			}

			Map<String, String> attributeNames = new HashMap<String, String>();
			for (Iterator<Locale> iter = attr.getDisplayNames().keySet().iterator(); iter.hasNext();) {
				Locale key = iter.next();
				LOG.trace(attr.getDisplayNames().get(key));
				attributeNames.put(key.getLanguage(), attr.getDisplayNames().get(key));
			}

			Map<String, String> attributeDescriptions = new HashMap<String, String>();
			for (Iterator<Locale> iter = attr.getDisplayDescriptions().keySet().iterator(); iter.hasNext();) {
				Locale key = iter.next();
				LOG.trace(attr.getDisplayDescriptions().get(key));
				attributeDescriptions.put(key.getLanguage(), attr.getDisplayDescriptions().get(key));
			}

			String attributeFriendlyName = null;
			List<AttributeEncoder> encoders = attr.getEncoders();
			for (int i = 0; i < encoders.size(); i++) {
				if (encoders.get(i) instanceof SAML2AttributeEncoder) {
					attributeFriendlyName = ((SAML2AttributeEncoder) encoders.get(i)).getFriendlyName();
				}
			}
			if (attributeFriendlyName == null) {
				attributeFriendlyName = attr.getId();
			}

			Attribute a = new Attribute();
			a.attributeID = attr.getId();
			a.attributeNames = attributeNames;
			a.attributeDescriptions = attributeDescriptions;
			a.attributeValues = attributeValues;
			a.attributeFriendlyName = attributeFriendlyName;

			result.add(a);
		}

		return result;
	}

}
