package ch.SWITCH.aai.uApprove.idpplugin;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.TreeSet;

import java.io.FileReader;
import java.io.BufferedReader;

import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

import ch.SWITCH.aai.uApprove.components.Attribute;
import ch.SWITCH.aai.uApprove.components.UApproveException;

public class AttributeList {

  /**
   * Comparator for ordering the attribute by "attribute-list" and alphabetical.
   */
  private static class WhiteListComparator implements Comparator<Attribute> {
    private List<String> whiteList;

    public WhiteListComparator(List<String> whiteLIst) {
      this.whiteList = whiteLIst;
    }

    @Override
    public int compare(Attribute o1, Attribute o2) {
      int i1 = whiteList.indexOf(o1.attributeID);
      int i2 = whiteList.indexOf(o2.attributeID);

      // Ordered by "attribute-list"
      if (i1 != -1 && i2 != -1) {
        if (i1 == i2) {
          return 0;
        }
        return (i1 > i2) ? 1 : -1;
      }

      // Attribute not found in the "attribute-list" is in alphabetical order.
      if (i1 == -1 && i2 == -1) {
        return o1.attributeID.compareTo(o2.attributeID);
      }

      // in the "attribute-list" < not in the "attribute-list"
      return (i1 == -1) ? 1 : -1;
    }
  }

  private static Comparator<Attribute> ca = null;

  private static AttributeList attributeList = null;
  private static List <String> blackList;
  private static List <String> whiteList;
  private static Logger LOG = LoggerFactory.getLogger(AttributeList.class);
  
  private AttributeList(String filename) {
    LOG.debug("AttributeList initialization from file="+filename);
    blackList = new ArrayList<String>();
    whiteList = new ArrayList<String>();
    try {
      if (filename == null) throw new UApproveException("AttributeList file is null");
      BufferedReader reader = new BufferedReader(new FileReader(filename));
      String line;
      while ((line = reader.readLine()) != null) {
        line.trim();
        if (line.startsWith("#") || line.startsWith("//")
            || line.length() == 0)
          continue;
        
        if (line.startsWith("!"))
          blackList.add(line.substring(1));
        else
          whiteList.add(line);
      }
      ca = new WhiteListComparator(whiteList);
    } catch (Exception e) {
      LOG.warn("Unable to load AttributeList file={}. AttributeList is initialized empty, Reason: {}", filename, e.getMessage());
    }
  }


  public static synchronized void initialize(String filename) {
    if (attributeList == null) {
    	attributeList = new AttributeList(filename);
    }
    
    LOG.debug("Attribute blacklist: {}", blackList);
    LOG.debug("Attribute sorted whitelist: {}", whiteList);
  }

  public static List <String> getWhiteList() {
      return whiteList;
  }
  
  public static boolean isBlackListed(String attr) {
    return blackList.contains(attr);
  }
  
  public static List<Attribute> sortAttributes(Collection<Attribute> unsortedAttributes) {
	  LOG.trace("Unsorted attributes: {}", getAttributeIds(unsortedAttributes));

	  List<Attribute> sortedAttributes = new ArrayList<Attribute>();
	  
	  for (Attribute attribute: unsortedAttributes) {
		  sortedAttributes.add(attribute);
	  }
	  Collections.sort(sortedAttributes, ca);

	  LOG.trace("Sorted attributes: {}", getAttributeIds(sortedAttributes));
	  return sortedAttributes;
  }
  
  public static Collection<String> getAttributeIds(Collection<Attribute> attributes) {
	  Collection<String> ids = new ArrayList<String>();
	  for (Attribute attribute: attributes) {
		  ids.add(attribute.attributeID);
	  }
	  return ids;
  }

  @Deprecated
  public static Collection<Attribute> removeBlacklistedAttributes(Collection<Attribute> allAttributes) {
	  LOG.trace("Attributes before blacklist removal: {}", getAttributeIds(allAttributes));
	  Collection<Attribute> attributes =
		  new TreeSet<Attribute>(new Comparator<Attribute>() {
			  public int compare(Attribute attr1, Attribute attr2) {
				  return attr1.attributeID.compareTo(attr2.attributeID);
			  }
		  });
	  for (Attribute attribute: allAttributes) {
		  if (!isBlackListed(attribute.attributeID)) {
			  attributes.add(attribute);
		  }
	  }
	  LOG.trace("Attributes after blacklist removal: {}", getAttributeIds(attributes));
	  return attributes;
  }
}
