/*
 * Copyright (c) 2010-2011, National Insutitute of Informatics
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.gakunin.shibboleth.common.config.attribute.filtering.match.uapprove;

import org.opensaml.xml.util.DatatypeHelper;
import org.opensaml.xml.util.XMLHelper;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import javax.xml.namespace.QName;
import org.w3c.dom.Element;

import edu.internet2.middleware.shibboleth.common.config.attribute.filtering.BaseFilterBeanDefinitionParser;

import jp.gakunin.shibboleth.common.attribute.filtering.provider.match.uapprove.AttributeUapproveMatchFunctor;

/**
 * Bean definition parser for {@link AttributeUapproveMatchFunctor}s.
 */
public class AttributeUapproveMatchFunctorBeanDefinitionParser extends
        BaseFilterBeanDefinitionParser {

    /** Schema type. */
    public static final QName SCHEMA_TYPE = new QName(UapproveMatchFunctorNamespaceHandler.NAMESPACE,
            "AttributeUapprove");
    
    /** {@inheritDoc} */
    protected Class getBeanClass(Element arg0) {
        return AttributeUapproveMatchFunctor.class;
    }

    /** {@inheritDoc} */
    protected void doParse(Element configElement, BeanDefinitionBuilder builder) {
        super.doParse(configElement, builder);

        boolean isApproved = true;
        if (configElement.hasAttributeNS(null, "isApproved")) {
            isApproved = XMLHelper.getAttributeValueAsBoolean(configElement.getAttributeNodeNS(null, "isApproved"));
        }
        builder.addPropertyValue("isApproved", isApproved);

        boolean requestedOnly = false;
        if (configElement.hasAttributeNS(null, "requestedOnly")) {
            requestedOnly = XMLHelper.getAttributeValueAsBoolean(configElement.getAttributeNodeNS(null, "requestedOnly"));
        }
        builder.addPropertyValue("requestedOnly", requestedOnly);
    }
    
}
